typedef struct phys_snd_dev
{
    char    name[64];
    int	    play_id;
    int	    rec_id;
} phys_snd_dev;


static pj_status_t enum_phys_snd_devs(pj_bool_t wmme, unsigned *dev_cnt, 
				      phys_snd_dev devs[])
{
    char *keyword[2] = { "Primary Sound ", "Microsoft Sound Mapper"};
    unsigned i, all_cnt, max;
    const pjmedia_snd_dev_info *di;
    unsigned start_rec, start_play;

    max = *dev_cnt;
    *dev_cnt = 0;

    all_cnt = pjmedia_snd_get_dev_count();

    /* Find the start of input devices */
    for (i=0; i<all_cnt; ++i) {
	di = pjmedia_snd_get_dev_info(i);
	if (strstr(di->name, keyword[wmme]) != 0) {
	    start_rec = i;
	    break;
	}
    }
    if (i==all_cnt)
	return PJ_ENOTFOUND;

    /* Make sure this is an input device */
    di = pjmedia_snd_get_dev_info(start_rec);
    PJ_ASSERT_RETURN(di->input_count != 0, PJ_EINVALIDOP);

    /* Find the start of output device */
    for (i=start_rec+1; i<all_cnt; ++i) {
	di = pjmedia_snd_get_dev_info(i);
	if (strstr(di->name, keyword[wmme]) != 0) {
	    start_play = i;
	    break;
	}
    }
    if (i==all_cnt)
	return PJ_ENOTFOUND;

    /* Make sure this is an output device */
    di = pjmedia_snd_get_dev_info(start_play);
    PJ_ASSERT_RETURN(di->output_count != 0, PJ_EINVALIDOP);

    /* Pair devices */
    for (i=start_rec; i<start_play; ++i) {
	unsigned j;

	di = pjmedia_snd_get_dev_info(i);
	strcpy(devs[*dev_cnt].name, di->name);
	devs[*dev_cnt].rec_id = i;

	/* Try to guess the output device index. */
	j = start_play+i-start_rec;
	di = pjmedia_snd_get_dev_info(j);
	if (strcmp(devs[*dev_cnt].name, di->name)==0) {
	    devs[*dev_cnt].play_id = j;
	    (*dev_cnt)++;
	    continue;
	}

	/* Find output device with matching name. This will not
	 * work with DirectSound if there are multiple sound 
	 * devices with the same name!
	 */
	for (j=start_play; j<all_cnt; ++j) {
	    di = pjmedia_snd_get_dev_info(j);
	    if (strcmp(devs[*dev_cnt].name, di->name)==0) {
		devs[*dev_cnt].play_id = j;
		break;
	    }
	}

	if (j != all_cnt)
	    (*dev_cnt)++;
    }

    return PJ_SUCCESS;
}

static void Setup_USBAudio()
{
    phys_snd_dev devs[32];
    unsigned i, count = PJ_ARRAY_SIZE(devs);
    pj_pool_t *pool;
    pj_status_t status;

    status = enum_phys_snd_devs(PJ_TRUE, &count, devs);
    if (status != PJ_SUCCESS) {
	PJ_LOG(1,(THIS_FILE, "Error enumerating physical sound devices"));
	return;
    }

    if (count == 0) {
	PJ_LOG(1,(THIS_FILE, "Error: no devices were found!"));
	return;
    }

    PJ_LOG(3,(THIS_FILE, "Found %d devices:", count));
    for (i=0; i<count; ++i) {
	PJ_LOG(3,(THIS_FILE, " %d. %s (play_id=%d, rec_id=%d)", i+1,
			     devs[i].name, devs[i].play_id, 
			     devs[i].rec_id));
    }


    /* Set the first device as the primary sound */
    pjsua_set_snd_dev(devs[0].rec_id, devs[0].play_id);

    pool = pjsua_pool_create("appsnd", 1000, 1000);

    /* Add remaining devices to the bridge */
    for (i=1; i<count; ++i) {
	unsigned	     Clock = 16000;
        pjmedia_port        *Splitter;
        pjmedia_port        *Port;
        pjmedia_snd_port    *SoundPort;
        pjsua_conf_port_id   ConferencePort;
        pj_str_t             ConferenceName;
	pj_status_t	     Status;

        Status = pjmedia_splitcomb_create(pool, Clock, 1, Clock/50, 16, 0, &Splitter);
        pj_assert(Status == PJ_SUCCESS);

        Status = pjmedia_snd_port_create(pool, devs[i].rec_id, devs[i].play_id, Clock, 1, Clock/50, 16, 0, &SoundPort);
        pj_assert(Status == PJ_SUCCESS);

        Status = pjmedia_snd_port_connect(SoundPort, Splitter);
        pj_assert(Status == PJ_SUCCESS);

        Status = pjmedia_splitcomb_create_rev_channel(pool, Splitter, 0, 0, &Port);
        pj_assert(Status == PJ_SUCCESS);

	ConferenceName = pj_str(devs[i].name);
        Status = pjmedia_conf_add_port(pjsua_var.mconf,pool, Port, &ConferenceName, &ConferencePort);
        pj_assert(Status == PJ_SUCCESS);

	PJ_LOG(3,(THIS_FILE, "Sound device '%s' added at slot %d",
		  devs[i].name, ConferencePort));
    }
}


